#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS GIS.include module

This module is used for Pyramid integration
"""

import re

from pyams_gis.interfaces import MANAGE_MAPS_PERMISSION, REST_AREA_TRANSFORM_PATH, REST_AREA_TRANSFORM_ROUTE, \
    REST_AREA_TRANSFORM_ROUTE_SETTING, REST_POINT_TRANSFORM_PATH, REST_POINT_TRANSFORM_ROUTE, \
    REST_POINT_TRANSFORM_ROUTE_SETTING
from pyams_security.interfaces.names import SYSTEM_ADMIN_ROLE

__docformat__ = 'restructuredtext'

from pyams_gis import _


def include_package(config):
    """Pyramid package include"""

    # add translations
    config.add_translation_dirs('pyams_gis:locales')

    # register permissions
    config.register_permission({
        'id': MANAGE_MAPS_PERMISSION,
        'title': _("Manage GIS maps")
    })

    # upgrade system manager role
    config.upgrade_role(SYSTEM_ADMIN_ROLE,
                        permissions={MANAGE_MAPS_PERMISSION})

    # register REST transform API routes
    config.add_route(REST_POINT_TRANSFORM_ROUTE,
                     config.registry.settings.get(REST_POINT_TRANSFORM_ROUTE_SETTING,
                                                  REST_POINT_TRANSFORM_PATH))
    config.add_route(REST_AREA_TRANSFORM_ROUTE,
                     config.registry.settings.get(REST_AREA_TRANSFORM_ROUTE_SETTING,
                                                  REST_AREA_TRANSFORM_PATH))

    try:
        import pyams_zmi  # pylint: disable=import-outside-toplevel,unused-import
    except ImportError:
        config.scan(ignore=[re.compile(r'pyams_gis\..*\.zmi\.?.*').search])
    else:
        config.scan()



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

import json
from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_gis import esri_leaflet, leaflet_google_mutant, leaflet_gp
from pyams_gis.interfaces.layer import IBaseTileMapLayer, IEsriFeatureMapLayer, IGeoJSONLayer, IGeoportalMapLayer, \
    IGoogleMapLayer, IMapLayer, ITileMapLayer, IWMSMapLayer, PYAMS_GIS_LAYERS_VOCABULARY
from pyams_gis.interfaces.utility import IMapManager
from pyams_i18n.interfaces import II18n
from pyams_utils.dict import update_dict
from pyams_utils.factory import factory_config
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.registry import get_utility
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'

from pyams_gis import _


@vocabulary_config(name=PYAMS_GIS_LAYERS_VOCABULARY)
class MapLayersVocabulary(SimpleVocabulary):
    """Map layers vocabulary"""

    def __init__(self, context):
        request = check_request()
        manager = get_utility(IMapManager)
        super().__init__([
            SimpleTerm(layer.__name__,
                       title=II18n(layer).query_attribute('title',
                                                          request=request))
            for layer in manager.values()
        ])


@implementer(IMapLayer)
class MapLayer(Persistent, Contained):
    """Base tile map layer persistent class"""

    factory = None
    depends = {}
    layer_type = None
    layer_icon = None

    name = FieldProperty(IMapLayer['name'])
    title = FieldProperty(IMapLayer['title'])
    min_zoom = FieldProperty(IMapLayer['min_zoom'])
    max_zoom = FieldProperty(IMapLayer['max_zoom'])
    is_overlay = FieldProperty(IMapLayer['is_overlay'])

    def get_configuration(self):
        """Get configuration mapping"""
        result = {
            'name': self.name,
            'title': II18n(self).query_attribute('title')
        }
        update_dict(result, 'factory', self.factory)
        update_dict(result, 'minZoom', self.min_zoom)
        update_dict(result, 'maxZoom', self.max_zoom)
        update_dict(result, 'isOverlay', self.is_overlay)
        if self.depends:
            depends = {}
            for name, resource in self.depends.items():
                depends[name] = get_resource_path(resource)
            update_dict(result, 'dependsOn', depends)
        return result


class GeoJSONLayer(MapLayer):
    """GeoJSON layer"""

    factory = 'MyAMS.gis.factory.GeoJSON'
    layer_type = _("GeoJSON")

    url = FieldProperty(IGeoJSONLayer['url'])
    style = FieldProperty(IGeoJSONLayer['style'])

    def get_configuration(self):
        result = super(GeoJSONLayer, self).get_configuration()
        update_dict(result, 'url', self.url)
        if self.style:
            update_dict(result, 'style', json.loads(self.style))
        return result


class BaseTileMapLayer(MapLayer):
    """Base tile map layer"""

    attribution = FieldProperty(IBaseTileMapLayer['attribution'])
    bounds = FieldProperty(IBaseTileMapLayer['bounds'])

    def get_configuration(self):
        result = super(BaseTileMapLayer, self).get_configuration()
        update_dict(result, 'attribution', self.attribution)
        if self.bounds:
            point1, point2 = self.bounds.wgs_coordinates
            result['bounds'] = [{
                'lat': float(point1[1]),
                'lon': float(point1[0])
            }, {
                'lat': float(point2[1]),
                'lon': float(point2[0])
            }]
        return result


@factory_config(ITileMapLayer)
class TileMapLayer(BaseTileMapLayer):
    """Base tile map layer persistent class"""

    factory = 'MyAMS.gis.factory.TileLayer'
    layer_type = _("Tile layer")
    layer_icon = 'fas fa-layer-group'

    url = FieldProperty(ITileMapLayer['url'])

    def get_configuration(self):
        result = super(TileMapLayer, self).get_configuration()
        update_dict(result, 'url', self.url)
        return result


@factory_config(IWMSMapLayer)
class WMSMapLayer(TileMapLayer):
    """WMS map mayer persistent class"""

    factory = 'MyAMS.gis.factory.WMS'
    layer_type = _("WMS layer")
    layer_icon = 'fas fa-map'

    crs = FieldProperty(IWMSMapLayer['crs'])
    layers = FieldProperty(IWMSMapLayer['layers'])
    styles = FieldProperty(IWMSMapLayer['styles'])
    format = FieldProperty(IWMSMapLayer['format'])
    transparent = FieldProperty(IWMSMapLayer['transparent'])
    version = FieldProperty(IWMSMapLayer['version'])
    uppercase = FieldProperty(IWMSMapLayer['uppercase'])

    def get_configuration(self):
        result = super(WMSMapLayer, self).get_configuration()
        update_dict(result, 'crs', self.crs)
        update_dict(result, 'layers', self.layers)
        update_dict(result, 'styles', self.styles)
        update_dict(result, 'format', self.format)
        update_dict(result, 'transparent', self.transparent)
        update_dict(result, 'version', self.version)
        update_dict(result, 'uppercase', self.uppercase)
        return result


@factory_config(IGeoportalMapLayer)
class GeoportalMapLayer(BaseTileMapLayer):
    """Geoportal map layer persistent class"""

    factory = 'MyAMS.gis.factory.Geoportal.WMS'
    depends = {
        'L.geoportalLayer.WMS': leaflet_gp
    }
    layer_type = _("GeoPortal layer")
    layer_icon = 'far fa-map'

    api_key = FieldProperty(IGeoportalMapLayer['api_key'])
    layer_name = FieldProperty(IGeoportalMapLayer['layer_name'])
    crs = FieldProperty(IGeoportalMapLayer['crs'])

    def get_configuration(self):
        result = super(GeoportalMapLayer, self).get_configuration()
        update_dict(result, 'apiKey', self.api_key)
        update_dict(result, 'layer', self.layer_name)
        update_dict(result, 'crs', self.crs)
        return result


@factory_config(IEsriFeatureMapLayer)
class EsriFeatureMapLayer(MapLayer):
    """ESRI feature map layer"""

    factory = 'MyAMS.gis.factory.ESRI.Feature'
    depends = {
        'L.esri.featureLayer': esri_leaflet
    }
    layer_type = _("ESRI features layer")
    layer_icon = 'fas fa-map-signs'

    url = FieldProperty(IEsriFeatureMapLayer['url'])
    token = FieldProperty(IEsriFeatureMapLayer['token'])
    where = FieldProperty(IEsriFeatureMapLayer['where'])

    def get_configuration(self):
        result = super(EsriFeatureMapLayer, self).get_configuration()
        update_dict(result, 'url', self.url)
        update_dict(result, 'token', self.token)
        update_dict(result, 'where', self.where)
        return result


@factory_config(IGoogleMapLayer)
class GoogleMapLayer(MapLayer):
    """Google maps layer"""

    factory = 'MyAMS.gis.factory.Google'
    depends = {
        'L.gridLayer.googleMutant': leaflet_google_mutant
    }
    layer_type = _("Google Maps layer")
    layer_icon = 'fas fa-map-marker'

    api_key = FieldProperty(IGoogleMapLayer['api_key'])
    type = FieldProperty(IGoogleMapLayer['type'])

    def get_configuration(self):
        result = super(GoogleMapLayer, self).get_configuration()
        update_dict(result, 'apiKey', self.api_key)
        update_dict(result, 'type', self.type)
        return result



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_gis.interfaces.configuration import IMapConfiguration, IMapConfigurationTarget, MAP_CONFIGURATION_KEY
from pyams_gis.interfaces.utility import IMapManager
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility


@factory_config(IMapConfiguration)
class MapConfiguration(Persistent, Contained):
    """Map configuration persistent class"""

    crs = FieldProperty(IMapConfiguration['crs'])
    layers = FieldProperty(IMapConfiguration['layers'])
    hidden_layers = FieldProperty(IMapConfiguration['hidden_layers'])
    auto_adjust = FieldProperty(IMapConfiguration['auto_adjust'])
    zoom_level = FieldProperty(IMapConfiguration['zoom_level'])
    initial_center = FieldProperty(IMapConfiguration['initial_center'])
    initial_bounds = FieldProperty(IMapConfiguration['initial_bounds'])
    attribution_control = FieldProperty(IMapConfiguration['attribution_control'])
    zoom_control = FieldProperty(IMapConfiguration['zoom_control'])
    fullscreen_control = FieldProperty(IMapConfiguration['fullscreen_control'])
    layer_control = FieldProperty(IMapConfiguration['layer_control'])
    keyboard = FieldProperty(IMapConfiguration['keyboard'])
    scroll_wheel_zoom = FieldProperty(IMapConfiguration['scroll_wheel_zoom'])

    def get_configuration(self):
        result = {
            'crs': self.crs,
            'layerControl': self.layer_control,
            'attributionControl': self.attribution_control,
            'zoomControl': self.zoom_control,
            'fullscreenControl': self.fullscreen_control,
            'keyboard': self.keyboard,
            'scrollWheelZoom': self.scroll_wheel_zoom,
            'zoom': self.zoom_level
        }
        if self.auto_adjust:
            result['adjust'] = 'auto'
        if self.initial_center:
            gps_location = self.initial_center.wgs_coordinates
            result['center'] = {
                'lon': float(gps_location['longitude']),
                'lat': float(gps_location['latitude'])
            }
        elif self.initial_bounds:
            point1, point2 = self.initial_bounds.wgs_coordinates
            result['bounds'] = [{
                'lon': float(point1[0]),
                'lat': float(point1[1])
            }, {
                'lon': float(point2[0]),
                'lat': float(point2[1])
            }]
        else:
            # Near center default location
            result['center'] = {
                'lat': 45,
                'lon': 5.0
            }
        manager = get_utility(IMapManager)
        layers = []
        if self.layers:
            for name in self.layers:
                layer = manager.get(name)
                if layer is not None:
                    configuration = layer.get_configuration()
                    configuration['isVisible'] = True
                    layers.append(configuration)
        if self.hidden_layers:
            for name in self.hidden_layers:
                layer = manager.get(name)
                if layer is not None:
                    configuration = layer.get_configuration()
                    configuration['isVisible'] = False
                    layers.append(configuration)
        result['layers'] = layers
        return result


@adapter_config(required=IMapConfigurationTarget,
                provides=IMapConfiguration)
def map_configuration(context):
    """Map configuration adapter"""
    return get_annotation_adapter(context, MAP_CONFIGURATION_KEY, IMapConfiguration)



#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_gis.interfaces module

This module defines main package interfaces.
"""

from enum import Enum

from zope.interface import Attribute, Interface, Invalid, invariant
from zope.schema import Choice
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_utils.list import is_not_none
from pyams_utils.schema import DottedDecimalField

from pyams_gis import _


REST_POINT_TRANSFORM_ROUTE = 'pyams_gis.rest.transform.point'
REST_POINT_TRANSFORM_ROUTE_SETTING = f'{REST_POINT_TRANSFORM_ROUTE}_route.path'
REST_POINT_TRANSFORM_PATH = '/api/gis/rest/transform/point'

REST_AREA_TRANSFORM_ROUTE = 'pyams_gis.rest.transform.area'
REST_AREA_TRANSFORM_ROUTE_SETTING = f'{REST_AREA_TRANSFORM_ROUTE}_route.path'
REST_AREA_TRANSFORM_PATH = '/api/gis/rest/transform/area'


#
# Custom permissions
#

MANAGE_MAPS_PERMISSION = 'pyams.ManageMaps'
'''Permission required to manage default maps configuration and layers'''


#
# Main maps CRS
#

class CRS(Enum):
    """Common CRS values"""
    WGS84 = 4326
    WGS84WM = 3857
    RGF93 = 2154
    LAMBERT_IIE = 27572
    UTM_20N = 4559
    UTM_22N = 2972
    UTM_38S = 4471
    UTM_40S = 2975


COORDINATES_PROJECTIONS = {
    CRS.WGS84.value: _("WGS84 (GPS)"),
    CRS.WGS84WM.value: _("WGS84 Web Mercator"),
    CRS.RGF93.value: _("Lambert 93 (Metropolitan France)"),
    CRS.LAMBERT_IIE.value: _("Extended Lambert II (Metropolitan France)"),
    CRS.UTM_20N.value: _("UTM Zone 20N (Martinique, Guadeloupe)"),
    CRS.UTM_22N.value: _("UTM Zone 22N (Guyane)"),
    CRS.UTM_38S.value: _("UTM Zone 38S (Mayotte)"),
    CRS.UTM_40S.value: _("UTM Zone 40S (La Réunion)")
}


COORDINATES_PROJECTION_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in COORDINATES_PROJECTIONS.items()
])


LAYER_CRS = {
    CRS.WGS84.value: 'L.CRS.EPSG4326',
    CRS.WGS84WM.value: 'L.CRS.EPSG3857',
    CRS.RGF93.value: 'L.geoportalCRS.EPSG2154',
    CRS.LAMBERT_IIE.value: 'L.geoportalCRS.EPSG27572'
}

LAYER_CRS_VOCABULARY = SimpleVocabulary([
    SimpleTerm(t, title=COORDINATES_PROJECTIONS[v])
    for v, t in LAYER_CRS.items()
])


class IGeoInfo(Interface):
    """Base geographic information interface"""

    def to_json(self):
        """Return JSON representation of current object"""


class IGeoPoint(IGeoInfo):
    """GeoPoint attribute interface"""

    longitude = DottedDecimalField(title=_("Longitude"),
                                   required=False)

    latitude = DottedDecimalField(title=_("Latitude"),
                                  required=False)

    projection = Choice(title=_("Projection system"),
                        vocabulary=COORDINATES_PROJECTION_VOCABULARY,
                        default=CRS.WGS84.value,
                        required=True)

    @invariant
    def check_coordinates(self):
        """Point coordinates checker"""
        data = tuple(map(is_not_none, (self.longitude, self.latitude)))
        if len(data) != 2:
            raise Invalid(_("You must set longitude and latitude, or None!"))
        if self.longitude and not self.projection:
            raise Invalid(_("You can't set coordinates without setting projection!"))

    def get_coordinates(self, projection=CRS.WGS84.value):
        """Get coordinates translated to given projection"""

    wgs_coordinates = Attribute("Coordinates tuple in WGS84 projection")


class IGeoPointZ(IGeoPoint):
    """GeoPointZ attribute interface"""

    altitude = DottedDecimalField(title=_("Altitude"),
                                  required=False)


class IGeoArea(IGeoInfo):
    """Geographic area defined by a rectangle"""

    x1 = DottedDecimalField(title=_("West limit"),
                            required=False)

    y1 = DottedDecimalField(title=_("South limit"),
                            required=False)

    x2 = DottedDecimalField(title=_("East limit"),
                            required=False)

    y2 = DottedDecimalField(title=_("North limit"),
                            required=False)

    projection = Choice(title=_("Projection system"),
                        vocabulary=COORDINATES_PROJECTION_VOCABULARY,
                        default=CRS.WGS84.value,
                        required=True)

    @invariant
    def check_coordinates(self):
        """Area coordinates checker"""
        data = tuple(map(is_not_none, (self.x1, self.x2, self.y1, self.y2)))
        if len(data) != 4:
            raise Invalid(_("You must set all coordinates or None!"))
        if self.x1 and not self.projection:
            raise Invalid(_("You can't set coordinates without setting projection!"))

    def get_coordinates(self, projection=CRS.WGS84.value):
        """Get coordinates translated to given projection"""

    wgs_coordinates = Attribute("Coordinates in WGS84 projection")



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_gis.interfaces.utility module

This module defines base maps manager interfaces.
"""

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.constraints import contains
from zope.container.interfaces import IContainer
from zope.interface import Interface
from zope.schema import Bool

from pyams_gis.interfaces.layer import IMapLayer

__docformat__ = 'restructuredtext'

from pyams_gis import _


class IMapManagerInfo(Interface):
    """Map manager information interface"""

    show_home_menu = Bool(title=_("Access menu from home"),
                          description=_("If 'yes', a menu will be displayed to get access to "
                                        "maps manager from site admin home page"),
                          required=True,
                          default=False)
    
    
class IMapManager(IMapManagerInfo, IContainer, IAttributeAnnotatable):
    """Map manager interface"""

    contains(IMapLayer)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.interface import Attribute
from zope.location.interfaces import IContained
from zope.schema import Bool, Choice, Int, Text, TextLine
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_gis.interfaces import LAYER_CRS_VOCABULARY
from pyams_gis.schema import GeoAreaField
from pyams_i18n.schema import I18nTextLineField

__docformat__ = 'restructuredtext'

from pyams_gis import _


#
# Leaflet map layers interfaces
#

PYAMS_GIS_LAYERS_VOCABULARY = 'pyams_gis.layers'


class IMapLayer(IContained, IAttributeAnnotatable):
    """Base map layer interface"""

    factory = Attribute("Layer factory name")

    depends = Attribute("List of layer factory dependent objects")

    name = TextLine(title=_("Layer name"),
                    required=True)

    layer_type = Attribute("Layer type")

    layer_icon = Attribute("Layer icon")

    title = I18nTextLineField(title=_("Layer title"),
                              description=_("Full layer title"),
                              required=True)

    min_zoom = Int(title=_("Minimum zoom level"),
                   description=_("Minimum zoom level at which layer is displayed"),
                   default=0,
                   required=True)

    max_zoom = Int(title=_("Maximum zoom level"),
                   description=_("Maximum zoom level at which layer is displayed"),
                   default=18,
                   required=True)

    is_overlay = Bool(title=_("Overlay layer?"),
                      description=_("Unlike base layers for which a single layer is visible at "
                                    "a given time, overlay layers are used to display multiple "
                                    "information on top a selected base layer"),
                      required=True,
                      default=False)

    def get_configuration(self):
        """Get layer configuration mapping"""


class IGeoJSONLayer(IMapLayer):
    """GeoJSON map layer interface"""

    url = TextLine(title=_("Layer URL"),
                   description=_("URL used to get access to JSON data"),
                   required=True)

    style = Text(title=_("Layer style"),
                 description=_("Layer style, provided in Leaflet JSON format"),
                 required=False)


class IBaseTileMapLayer(IMapLayer):
    """Base tiles map layer interface"""

    attribution = TextLine(title=_("Layer attribution"),
                           description=_("String used by the attribution control"),
                           required=False)

    bounds = GeoAreaField(title=_("Layer bounds"),
                          description=_("Geographical bounds into which layer tiles are displayed"),
                          required=False)


class ITileMapLayer(IBaseTileMapLayer):
    """Tiles map layer interface"""

    url = TextLine(title=_("URL template"),
                   description=_("URL template used to get layer tiles (see leaflet.js docs)"),
                   default='http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
                   required=True)


class IWMSMapLayer(ITileMapLayer):
    """WMS map layer interface"""

    crs = Choice(title=_("CRS"),
                 description=_("Coordinates reference system to use for map requests; defaults to map request"),
                 vocabulary=LAYER_CRS_VOCABULARY,
                 required=False)

    layers = TextLine(title=_("Layers"),
                      description=_("Comma-separated list of WMS layers to show"),
                      required=False)

    styles = TextLine(title=_("Styles"),
                      description=_("Comma-separated list of WMS styles"),
                      required=False)

    format = TextLine(title=_("Layer format"),
                      description=_("WMS image format; use 'image/png' for layers with transparency"),
                      required=True,
                      default='image/png')

    transparent = Bool(title=_("Transparent?"),
                       description=_("If 'yes', the WMS services will return images with transparency"),
                       required=True,
                       default=False)

    version = TextLine(title=_("Version"),
                       description=_("Version of the WMS service to use"),
                       required=True,
                       default='1.1.1')

    uppercase = Bool(title=_("Uppercase?"),
                     description=_("If 'yes', WMS request parameters keys will be uppercase"),
                     required=True,
                     default=False)


class IGeoportalMapLayer(IBaseTileMapLayer):
    """French IGN map layer interface"""

    api_key = TextLine(title=_("API key"),
                       description=_("Key used to access layer data"),
                       required=True)

    layer_name = TextLine(title=_("IGN layer name"),
                          description=_("Name of layer in IGN format"),
                          required=True)

    crs = Choice(title=_("CRS"),
                 description=_("Coordinates reference system to use for map requests; defaults to map request"),
                 vocabulary=LAYER_CRS_VOCABULARY,
                 required=False)


class IEsriBaseMapLayer(ITileMapLayer):
    """ESRI map layer interface"""


class IEsriFeatureMapLayer(IMapLayer):
    """ESRI feature layer interface"""

    url = TextLine(title=_("Layer URL"),
                   description=_("URL used to get the feature layer"),
                   required=True)

    token = TextLine(title=_("Token"),
                     description=_("Token used in all service requests"),
                     required=False)

    where = TextLine(title=_("Where condition"),
                     description=_("Optional expression used to filter features"),
                     required=False)


GOOGLE_MAP_TYPES = {'roadmap': _("Roads map"),
                    'satellite': _("Satellite"),
                    'hybrid': _("Hybrid"),
                    'terrain': _("Terrain")}


GOOGLE_MAP_TYPES_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in GOOGLE_MAP_TYPES.items()
])


class IGoogleMapLayer(IMapLayer):
    """Google Maps layer"""

    api_key = TextLine(title=_("API key"),
                       description=_("Google API key used to access maps data"),
                       required=True)

    type = Choice(title=_("Map type"),
                  description=_("Type of GoogleMaps layer type"),
                  vocabulary=GOOGLE_MAP_TYPES_VOCABULARY,
                  default='roadmap',
                  required=True)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_gis.transform module

This module provides a single function which is used to convert coordinates
from one SRID to another one.
"""

__docformat__ = 'restructuredtext'

try:
    from osgeo.osr import CoordinateTransformation, OAMS_TRADITIONAL_GIS_ORDER, SpatialReference
    have_gdal = True
except ImportError:
    have_gdal = False


def transform(point, from_srid, to_srid):
    """Transform point coordinates from source projection to another projection

    :param point: source point coordinates; can be given as a (longitude, latitude) tuple
        or as a mapping containing both keys
    :param from_srid: source coordinates system given as SRID
    :param to_srid: target coordinates system given as SRID
    :return: mapping with new 'point' coordinates containing transformed coordinates, and 'projection'
        key containing SRID of result projection system
    """
    longitude = None
    latitude = None
    if isinstance(point, (list, tuple)):
        longitude, latitude = map(float, point)
    elif isinstance(point, dict):
        longitude = float(point['longitude'])
        latitude = float(point['latitude'])
    from_srid = int(from_srid)
    to_srid = int(to_srid)
    if (not have_gdal) or (from_srid == to_srid):
        return {
            'point': {
                'longitude': longitude,
                'latitude': latitude
            },
            'srid': from_srid
        }
    source = SpatialReference()
    source.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER)
    source.ImportFromEPSG(from_srid)
    destination = SpatialReference()
    destination.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER)
    destination.ImportFromEPSG(to_srid)
    transformed = CoordinateTransformation(source, destination).TransformPoint(longitude, latitude)
    return {
        'point': {
            'longitude': transformed[0],
            'latitude': transformed[1]
        },
        'srid': to_srid
    }



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from zope.interface import Interface

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_gis.interfaces import MANAGE_MAPS_PERMISSION
from pyams_gis.interfaces.utility import IMapManager, IMapManagerInfo
from pyams_gis.zmi.interfaces import IMapManagerMenu
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_gis import _


@adapter_config(required=(IMapManager, IAdminLayer, Interface),
                provides=IObjectLabel)
def map_manager_label(context, request, view):
    """Maps manager label getter"""
    return request.localizer.translate(_("Maps manager"))


@viewlet_config(name='map-manager-properties.menu',
                context=IMapManager, layer=IAdminLayer,
                manager=IMapManagerMenu, weight=20,
                permission=MANAGE_MAPS_PERMISSION)
class MapManagerPropertiesMenu(NavigationMenuItem):
    """Map manager properties menu"""
    
    label = _("Properties")
    href = '#map-manager-properties.html'
    
    
@ajax_form_config(name='map-manager-properties.html',
                  context=IMapManager, layer=IPyAMSLayer,
                  permission=MANAGE_MAPS_PERMISSION)
class  MapManagerPropertiesEditForm(AdminEditForm):
    """Maps manager properties edit form"""
    
    title = _("Map manager properties")
    legend = _("Management interface")
    
    fields = Fields(IMapManagerInfo)
    


#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS GIS package

PyAMS GIS extension package
"""

__docformat__ = 'restructuredtext'

from fanstatic import Library, Resource
from pyramid.i18n import TranslationStringFactory

from pyams_utils.fanstatic import ExternalResource

_ = TranslationStringFactory('pyams_gis')


library = Library('pyams_gis', 'resources')

leaflet_css = Resource(library, 'css/leaflet-1.9.4.css',
                       minified='css/leaflet-1.9.4.min.css')

leaflet = Resource(library, 'js/leaflet-1.9.4.js',
                   minified='js/leaflet-1.9.4.min.js',
                   depends=[leaflet_css, ],
                   bottom=True)

esri_leaflet_gp = Resource(library, 'js/esri-leaflet-gp-3.0.0.js',
                           depends=[leaflet, ],
                           bottom=True)

esri_leaflet = Resource(library, 'js/esri-leaflet-3.0.12.js',
                        depends=[leaflet, ],
                        bottom=True)

leaflet_google = ExternalResource(library, 'https://maps.googleapis.com/maps/api/js',
                                  bottom=True)

leaflet_gp = Resource(library, 'js/GpPluginLeaflet.js',
                      minified='js/GpPluginLeaflet.min.js',
                      depends=[leaflet, ],
                      bottom=True)

leaflet_google_mutant = Resource(library, 'js/Leaflet.GoogleMutant.js',
                                 minified='js/Leaflet.GoogleMutant.min.js',
                                 depends=[leaflet, ],
                                 bottom=True)

pyams_gis = Resource(library, 'js/pyams_gis.js',
                     minified='js/pyams_gis.min.js',
                     depends=[leaflet, ],
                     bottom=True)


def includeme(config):
    """pyams_gis features include"""
    from .include import include_package  # pylint: disable=import-outside-toplevel
    include_package(config)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from decimal import Decimal

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty

from pyams_gis.interfaces import CRS, IGeoArea
from pyams_gis.transform import transform
from pyams_utils.factory import factory_config
from pyams_utils.list import is_not_none

__docformat__ = 'restructuredtext'


@factory_config(IGeoArea)
class GeoArea(Persistent):
    """GeoArea attribute object"""

    x1 = FieldProperty(IGeoArea['x1'])
    y1 = FieldProperty(IGeoArea['y1'])
    x2 = FieldProperty(IGeoArea['x2'])
    y2 = FieldProperty(IGeoArea['y2'])
    projection = FieldProperty(IGeoArea['projection'])

    def __init__(self, data=None, **kwargs):
        super().__init__()
        if 'x1' in kwargs:
            self.x1 = Decimal(kwargs['x1'])
        if 'y1' in kwargs:
            self.y1 = Decimal(kwargs['y1'])
        if 'x2' in kwargs:
            self.x2 = Decimal(kwargs['x2'])
        if 'y2' in kwargs:
            self.y2 = Decimal(kwargs['y2'])
        if 'projection' in kwargs:
            self.projection = kwargs['projection']

    def __bool__(self):
        return len(tuple(filter(is_not_none, (self.x1, self.y1, self.x2, self.y2)))) == 4

    def get_coordinates(self, projection=CRS.WGS84.value):
        if projection == self.projection:
            return (self.x1, self.y1), (self.x2, self.y2)
        point1 = transform({
            'longitude': float(self.x1),
            'latitude': float(self.y1)
        }, self.projection, projection).get('point')
        point2 = transform({
            'longitude': float(self.x2),
            'latitude': float(self.y2)
        }, self.projection, projection).get('point')
        return (
            (point1['longitude'], point1['latitude']),
            (point2['longitude'], point2['latitude'])
        )

    @property
    def wgs_coordinates(self):
        return self.get_coordinates(CRS.WGS84.value)

    def to_json(self):
        if not self:
            return None
        return {
            'x1': float(self.x1),
            'y1': float(self.y1),
            'x2': float(self.x2),
            'y2': float(self.y2),
            'crs': self.projection
        }



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from cornice import Service
from cornice.validators import colander_validator
from pyramid.httpexceptions import HTTPOk
from pyramid.testing import DummyRequest

from pyams_gis.api.schema import GeoAreaTransformRequest, GeoAreaTransformResponse, GeoPointTransformRequest, \
    GeoPointTransformResponse
from pyams_gis.interfaces import REST_AREA_TRANSFORM_ROUTE, REST_POINT_TRANSFORM_ROUTE
from pyams_gis.transform import have_gdal, transform
from pyams_security.interfaces.base import USE_INTERNAL_API_PERMISSION
from pyams_security.rest import check_cors_origin, set_cors_headers
from pyams_utils.rest import rest_responses

__docformat__ = 'restructuredtext'


#
# Point transform service
#

point_transform_service = Service(name=REST_POINT_TRANSFORM_ROUTE,
                                  pyramid_route=REST_POINT_TRANSFORM_ROUTE,
                                  description="PyAMS GIS point transformation service")


@point_transform_service.options(validators=(check_cors_origin, set_cors_headers))
def point_transform_options(request):
    """Point transformation options endpoint"""
    return ''


point_transform_responses = rest_responses.copy()
point_transform_responses[HTTPOk.code] = GeoPointTransformResponse(
    description="Point transformation response")


@point_transform_service.post(schema=GeoPointTransformRequest,
                              validators=(check_cors_origin, colander_validator, set_cors_headers),
                              response_schemas=point_transform_responses,
                              permission=USE_INTERNAL_API_PERMISSION,
                              require_csrf=False)
def transform_point(request):
    """Point transformation request handler"""
    params = request.params if isinstance(request, DummyRequest) else request.validated.get('body')
    result = {
        'status': 'success'
    }
    result.update(transform(**params))
    return result


#
# Area transform service
#

area_transform_service = Service(name=REST_AREA_TRANSFORM_ROUTE,
                                 pyramid_route=REST_AREA_TRANSFORM_ROUTE,
                                 description="PyAMS GIS area transformation service")


@area_transform_service.options(validators=(check_cors_origin, set_cors_headers))
def area_transform_options(request):
    """Area transformation options endpoint"""
    return ''


area_transform_responses = rest_responses.copy()
area_transform_responses[HTTPOk.code] = GeoAreaTransformResponse(
    description="Area transformation response")


@area_transform_service.post(schema=GeoAreaTransformRequest,
                             validators=(check_cors_origin, colander_validator, set_cors_headers),
                             response_schemas=area_transform_responses,
                             permission=USE_INTERNAL_API_PERMISSION,
                             require_csrf=False)
def transform_area(request):
    """Area transformation request handler"""
    params = request.params if isinstance(request, DummyRequest) else request.validated.get('body')
    result = {
        'status': 'success'
    }
    area = params.get('area')
    from_srid = params.get('from_srid')
    to_srid = params.get('to_srid')
    if (not have_gdal) or (from_srid == to_srid):
        result.update({
            'area': {
                'x1': float(area['x1']),
                'y1': float(area['y1']),
                'x2': float(area['x2']),
                'y2': float(area['y2'])
            },
            'srid': to_srid
        })
        return result
    point1 = transform((area['x1'], area['y1']), from_srid, to_srid).get('point')
    point2 = transform((area['x2'], area['y2']), from_srid, to_srid).get('point')
    result.update({
        'area': {
            'x1': point1['longitude'],
            'y1': point1['latitude'],
            'x2': point2['longitude'],
            'y2': point2['latitude']
        },
        'srid': to_srid
    })
    return result



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.container.folder import Folder
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_gis.interfaces.configuration import IMapConfigurationTarget
from pyams_gis.interfaces.layer import PYAMS_GIS_LAYERS_VOCABULARY
from pyams_gis.interfaces.utility import IMapManager
from pyams_i18n.interfaces import II18n
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'


@factory_config(IMapManager)
@implementer(IMapConfigurationTarget)
class MapManager(Folder):
    """Map manager utility"""
    
    show_home_menu = FieldProperty(IMapManager['show_home_menu'])


@vocabulary_config(name=PYAMS_GIS_LAYERS_VOCABULARY)
class MapLayersVocabulary(SimpleVocabulary):
    """Map manager layers vocabulary"""

    def __init__(self, context):
        request = check_request()
        manager = get_utility(IMapManager)
        super().__init__([
            SimpleTerm(layer.__name__,
                       title=II18n(layer).query_attribute('title', request=request))
            for layer in manager.values()
        ])



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

import json
from zope.interface import Interface, alsoProvides, implementer_only

from pyams_form.browser.object import ObjectWidget
from pyams_form.interfaces import IObjectFactory
from pyams_form.interfaces.form import IForm
from pyams_form.interfaces.widget import IFieldWidget, IObjectWidget
from pyams_form.widget import FieldWidget
from pyams_gis.interfaces import IGeoPoint, IGeoPointZ
from pyams_gis.interfaces.widget import IGeoPointWidget, IGeoPointZWidget
from pyams_gis.schema import IGeoPointField, IGeoPointZField
from pyams_layer.interfaces import IFormLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import get_interface_name, get_object_factory
from pyams_utils.interfaces.data import IObjectData


#
# GeoPoint widget
#

@adapter_config(name=get_interface_name(IGeoPoint),
                required=(Interface, IFormLayer, IForm, IGeoPointWidget),
                provides=IObjectFactory)
@adapter_config(name=get_interface_name(IGeoPoint),
                required=(Interface, IFormLayer, IObjectWidget, IGeoPointWidget),
                provides=IObjectFactory)
def geo_point_object_widget_factory(*args):  # pylint: disable=unused-argument
    """GeoPoint object factory"""
    return get_object_factory(IGeoPoint)


@implementer_only(IGeoPointWidget)
class GeoPointWidget(ObjectWidget):
    """GeoPoint widget"""

    def update_widgets(self, set_errors=True):
        super().update_widgets(set_errors)
        widgets = self.widgets
        for name in ('longitude', 'latitude'):
            widget = widgets.get(name)
            if widget is not None:
                widget.readonly = 'readonly'
                widget.label_css_class = 'control-label col-md-3'
                widget.input_css_class = 'col-md-4'
        projection = widgets.get('projection')
        if projection is not None:
            projection.label_css_class = 'control-label col-md-3'
            projection.input_css_class = 'col-md-4'
            projection.object_data = {
                'ams-events-handlers': {
                    'select2:selecting': 'MyAMS.gis.position.beforeProjectionChange',
                    'change.select2': 'MyAMS.gis.position.changedProjection'
                }
            }
            alsoProvides(projection, IObjectData)

    @property
    def wgs_coordinates(self):
        value = self.field.get(self.field.interface(self.context))
        if not value:
            return json.dumps({
                'longitude': None,
                'latitude': None
            })
        point = value.wgs_coordinates
        return json.dumps({
            'longitude': float(point[0]),
            'latitude': float(point[1])
        })


@adapter_config(required=(IGeoPointField, IFormLayer),
                provides=IFieldWidget)
def GeoPointFieldWidget(field, request):
    """GeoPoint field widget factory"""
    return FieldWidget(field, GeoPointWidget(request))


#
# GeoPointZ widget
#

@adapter_config(name=get_interface_name(IGeoPointZ),
                required=(Interface, IFormLayer, IForm, IGeoPointZWidget),
                provides=IObjectFactory)
def geo_pointz_object_widget_factory(context, request, form, widget):
    """GeoPointZ object factory"""
    return get_object_factory(IGeoPointZ)


@implementer_only(IGeoPointZWidget)
class GeoPointZWidget(GeoPointWidget):
    """GeoPointZ widget"""

    def update_widgets(self, set_errors=True):
        super().update_widgets(set_errors)
        altitude = self.widgets.get('altitude')
        if altitude is not None:
            altitude.label_css_class = 'control-label col-md-3'
            altitude.input_css_class = 'col-md-2'

    @property
    def wgs_coordinates(self):
        value = self.field.get(self.field.interface(self.context))
        if not value:
            return json.dump({
                'longitude': None,
                'latitude': None,
                'altitude': None
            })
        point = value.wgs_coordinates
        return json.dumps({
            'longitude': float(point[0]),
            'latitude': float(point[1]),
            'altitude': value.altitude
        })


@adapter_config(required=(IGeoPointZField, IFormLayer),
                provides=IFieldWidget)
def GeoPointZFieldWidget(field, request):
    """GeoPointZ field widget factory"""
    return FieldWidget(field, GeoPointZWidget(request))



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

import json
from zope.interface import Interface, alsoProvides, implementer_only

from pyams_form.browser.object import ObjectWidget
from pyams_form.interfaces import IObjectFactory
from pyams_form.interfaces.form import IForm
from pyams_form.interfaces.widget import IFieldWidget, IObjectWidget
from pyams_form.widget import FieldWidget
from pyams_gis.interfaces import IGeoArea
from pyams_gis.interfaces.widget import IGeoAreaWidget
from pyams_gis.schema import IGeoAreaField
from pyams_layer.interfaces import IFormLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import get_interface_name, get_object_factory
from pyams_utils.interfaces.data import IObjectData


#
# GeoArea widget
#

@adapter_config(name=get_interface_name(IGeoArea),
                required=(Interface, IFormLayer, IForm, IGeoAreaWidget),
                provides=IObjectFactory)
@adapter_config(name=get_interface_name(IGeoArea),
                required=(Interface, IFormLayer, IObjectWidget, IGeoAreaWidget),
                provides=IObjectFactory)
def geo_area_object_widget_factory(*args):  # pylint: disable=unused-argument
    """GeoArea object factory"""
    return get_object_factory(IGeoArea)


@implementer_only(IGeoAreaWidget)
class GeoAreaWidget(ObjectWidget):
    """GeoArea widget"""
    
    def update_widgets(self, set_errors=True):
        super().update_widgets(set_errors)
        widgets = self.widgets
        for name in ('x1', 'y1', 'x2', 'y2'):
            widget = widgets.get(name)
            if widget is not None:
                widget.readonly = 'readonly'
                widget.label_css_class = 'control-label col-md-3'
                widget.input_css_class = 'col-md-4'
        projection = widgets.get('projection')
        if projection is not None:
            projection.label_css_class = 'control-label col-md-3'
            projection.input_css_class = 'col-md-4'
            projection.object_data = {
                'ams-events-handlers': {
                    'select2:selecting': 'MyAMS.gis.area.beforeProjectionChange',
                    'change.select2': 'MyAMS.gis.area.changedProjection'
                }
            }
            alsoProvides(projection, IObjectData)
    
    @property
    def wgs_coordinates(self):
        value = self.field.get(self.field.interface(self.context))
        if not value:
            return json.dumps({
                'x1': None,
                'y1': None,
                'x2': None,
                'y2': None
            })
        point1, point2 = value.wgs_coordinates
        return json.dumps({
            'x1': float(point1[0]),
            'y1': float(point1[1]),
            'x2': float(point2[0]),
            'y2': float(point2[1])
        })


@adapter_config(required=(IGeoAreaField, IFormLayer),
                provides=IFieldWidget)
def GeoAreaFieldWidget(field, request):
    """GeoArea field widget factory"""
    return FieldWidget(field, GeoAreaWidget(request))



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""
from pyams_form.interfaces import INPUT_MODE
from pyams_form.interfaces.widget import IObjectWidget
from pyams_form.template import widget_template_config
from pyams_layer.interfaces import IFormLayer, IPyAMSLayer

__docformat__ = 'restructuredtext'


@widget_template_config(mode=INPUT_MODE,
                        template='templates/geopoint-input.pt',
                        layer=IPyAMSLayer)
class IGeoPointWidget(IObjectWidget):
    """GeoPoint widget interface"""


class IGeoPointZWidget(IGeoPointWidget):
    """GeoPointZ widget interface"""


@widget_template_config(mode=INPUT_MODE,
                        template='templates/geoarea-input.pt',
                        layer=IPyAMSLayer)
class IGeoAreaWidget(IObjectWidget):
    """GeoArea widget interface"""
